-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCastBar")
local SML = AceLibrary("SharedMedia-1.0")
local SML_statusbar = SML:List("statusbar")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Cast Bar"] = true,
	["Cast bar options"] = true,
	["Enable"] = true,
	["Show cast bars"] = true,
	["Hide Expired"] = true,
	["Forces cast bars to disappear when a unit is first selected if they aren't casting anything"] = true,
	["Position"] = true,
	["Adjust cast bar position"] = true,
	["Left Offset"] = true,
	["Sets the offset of the left of the cast bar"] = true,
	["Right Offset"] = true,
	["Sets the offset of the right of the cast bar"] = true,
	["Vertical Offset"] = true,
	["Sets the vertical offset of the cast bar"] = true,
	["Texture"] = true,
	["Sets the cast bar texture"] = true,
	["Height"] = true,
	["Sets the height of the cast bar"] = true,
} end)

-----------------------------------------------------------------------------

local AloftCastBar = Aloft:NewModule("CastBar", "AceEvent-2.0")
AloftCastBar.db = Aloft:AcquireDBNamespace("castBar")
Aloft:RegisterDefaults("castBar", "profile", {
	offsets =
	{
		left		= 12,
		right		= 0,
		vertical	= -2,
	},
	hideExpired		= true,
	height			= 10,
	texture			= "Blizzard"
})

Aloft.Options.args.castBar =
{
	type = 'group',
	name = L["Cast Bar"],
	desc = L["Cast bar options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Show cast bars"],
			get = function() return GetCVar("ShowVKeyCastbar") == "1" end,
			set = function(v) SetCVar("ShowVKeyCastbar", v and "1" or "0") end,
			order = 1
		},
		hideExpired =
		{
			type = 'toggle',
			name = L["Hide Expired"],
			desc = L["Forces cast bars to disappear when a unit is first selected if they aren't casting anything"],
			disabled = function() return GetCVar("ShowVKeyCastbar") ~= "1" end,
			get = function() return AloftCastBar.db.profile.hideExpired end,
			set = function(v) AloftCastBar.db.profile.hideExpired = v end,
		},
		position =
		{
			type = 'group',
			name = L["Position"],
			desc = L["Adjust cast bar position"],
			disabled = function() return GetCVar("ShowVKeyCastbar") ~= "1" end,
			args =
			{
				left =
				{
					type = 'range',
					name = L["Left Offset"],
					desc = L["Sets the offset of the left of the cast bar"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBar.db.profile.offsets.left end,
					set = function(v) AloftCastBar.db.profile.offsets.left = v AloftCastBar:UpdateAll() end
				},
				right =
				{
					type = 'range',
					name = L["Right Offset"],
					desc = L["Sets the offset of the right of the cast bar"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBar.db.profile.offsets.right end,
					set = function(v) AloftCastBar.db.profile.offsets.right = v AloftCastBar:UpdateAll() end
				},
				vertical =
				{
					type = 'range',
					name = L["Vertical Offset"],
					desc = L["Sets the vertical offset of the cast bar"],
					min = -32,
					max = 32,
					step = 1,
					get = function() return AloftCastBar.db.profile.offsets.vertical end,
					set = function(v) AloftCastBar.db.profile.offsets.vertical = v AloftCastBar:UpdateAll() end
				},
			},
		},
		height =
		{
			type = 'range',
			name = L["Height"],
			desc = L["Sets the height of the cast bar"],
			min = 5,
			max = 32,
			step = 1,
			disabled = function() return GetCVar("ShowVKeyCastbar") ~= "1" end,
			get = function() return AloftCastBar.db.profile.height end,
			set = function(v) AloftCastBar.db.profile.height = v AloftCastBar:UpdateAll() end
		},
		texture =
		{
			type = "text", 
			name = L["Texture"], 
			desc = L["Sets the cast bar texture"],
			disabled = function() return GetCVar("ShowVKeyCastbar") ~= "1" end,
			get = function() return AloftCastBar.db.profile.texture end,
			set = function(v) AloftCastBar.db.profile.texture = v AloftCastBar:UpdateAll() end,
			validate = SML_statusbar,
		},
	},
}

-----------------------------------------------------------------------------

function AloftCastBar:UpdateAll()
	for frame in Aloft:IterateNameplates() do
		self:SetupFrame(frame)

		if frame:IsShown() then
			self:OnCastBarShow(frame)
		end
	end
end

-----------------------------------------------------------------------------

function AloftCastBar:OnEnable()
	self:RegisterEvent("Aloft:SetupFrame", "SetupFrame")
	self:RegisterEvent("Aloft:OnCastBarShow", "OnCastBarShow")

	self:RegisterEvent("SharedMedia_SetGlobal", function(mediatype, override)
		if mediatype == "statusbar" then
			self:UpdateAll()
		end
	end)

	self:RegisterEvent("Aloft:SetAll", function(type, value)
		if self.db.profile[type] then
			self.db.profile[type] = value
			self:UpdateAll()
		end
	end)
	
	self:UpdateAll()
end

function AloftCastBar:OnDisable()
	self:UnregisterAllEvents()
	
	for frame in Aloft:IterateNameplates() do
		local castBar = frame.aloftData.castBar
	
		castBar:ClearAllPoints()
		castBar:SetPoint("BOTTOMRIGHT", select(2, frame:GetRegions()), "BOTTOMRIGHT", -4.5, 4.5)
		castBar:SetWidth(116.5)
		castBar:SetHeight(10.18)
		castBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
	end
end

function AloftCastBar:SetupFrame(frame)
	local castBar = frame.aloftData.castBar
	castBar:SetStatusBarTexture(SML:Fetch("statusbar", self.db.profile.texture))
end

function AloftCastBar:OnCastBarShow(frame)
	local castBar = frame.aloftData.castBar
	local healthBar = frame.aloftData.healthBar
	local profile = self.db.profile

	castBar:ClearAllPoints()
	castBar:SetPoint("TOPLEFT", healthBar, "BOTTOMLEFT", profile.offsets.left, profile.offsets.vertical)
	castBar:SetPoint("BOTTOMRIGHT", healthBar, "BOTTOMRIGHT", profile.offsets.right, profile.offsets.vertical - profile.height)
	castBar:SetAlpha(1.0)
end

-----------------------------------------------------------------------------
